@@ -1,5 +1,7 @@
 #!/bin/bash
 
+export MISTRAL_PATH=${DEPS_PATH}/mistral
+
 function get_dependencies {
     # Fetch mistral
     mkdir -p ${MISTRAL_PATH}
@@ -17,6 +19,10 @@ function build_nextpnr {
     popd
 }
 
+function run_tests {
+    :
+}
+
 function run_archcheck {
     pushd build
     ./nextpnr-mistral --device 5CEBA2F17A7 --test
