@@ -34,6 +34,8 @@
 #include <fstream>
 #include <iostream>
 #include <random>
+#include <set>
+
 #include "command.h"
 #include "design_utils.h"
 #include "json_frontend.h"
@@ -84,14 +86,14 @@ bool CommandHandler::parseOptions()
 bool CommandHandler::executeBeforeContext()
 {
     if (vm.count("help") || argc == 1) {
-        std::cerr << boost::filesystem::basename(argv[0])
+        std::cerr << boost::filesystem::path(argv[0]).stem()
                   << " -- Next Generation Place and Route (Version " GIT_DESCRIBE_STR ")\n";
         std::cerr << options << "\n";
         return argc != 1;
     }
 
     if (vm.count("version")) {
-        std::cerr << boost::filesystem::basename(argv[0])
+        std::cerr << boost::filesystem::path(argv[0]).stem()
                   << " -- Next Generation Place and Route (Version " GIT_DESCRIBE_STR ")\n";
         return true;
     }
@@ -103,6 +105,10 @@ bool CommandHandler::executeBeforeContext()
         log_streams.push_back(std::make_pair(&std::cerr, LogLevel::LOG_MSG));
     }
 
+    if (vm.count("Werror")) {
+        log_warn_as_error = true;
+    }
+
     if (vm.count("log")) {
         std::string logfilename = vm["log"].as<std::string>();
         logfile.open(logfilename);
@@ -119,6 +125,7 @@ po::options_description CommandHandler::getGeneralOptions()
     general.add_options()("help,h", "show help");
     general.add_options()("verbose,v", "verbose output");
     general.add_options()("quiet,q", "quiet mode, only errors and warnings displayed");
+    general.add_options()("Werror", "Turn warnings into errors");
     general.add_options()("log,l", po::value<std::string>(),
                           "log file, all log messages are written to this file regardless of -q");
     general.add_options()("debug", "debug output");
@@ -172,6 +179,7 @@ po::options_description CommandHandler::getGeneralOptions()
     general.add_options()("no-pack", "process design without packing");
 
     general.add_options()("ignore-loops", "ignore combinational loops in timing analysis");
+    general.add_options()("ignore-rel-clk", "ignore clock-to-clock relations in timing checks");
 
     general.add_options()("version,V", "show version");
     general.add_options()("test", "check architecture database integrity");
@@ -188,8 +196,11 @@ po::options_description CommandHandler::getGeneralOptions()
     general.add_options()("placer-heap-critexp", po::value<int>(),
                           "placer heap criticality exponent (int, default: 2)");
     general.add_options()("placer-heap-timingweight", po::value<int>(), "placer heap timing weight (int, default: 10)");
+    general.add_options()("placer-heap-cell-placement-timeout", po::value<int>(),
+                          "allow placer to attempt up to max(10000, total cells^2 / N) iterations to place a cell (int "
+                          "N, default: 8, 0 for no timeout)");
 
-#if !defined(__wasm)
+#if !defined(NPNR_DISABLE_THREADS)
     general.add_options()("parallel-refine", "use new experimental parallelised engine for placement refinement");
 #endif
 
@@ -252,7 +263,9 @@ void CommandHandler::setupContext(Context *ctx)
     if (vm.count("randomize-seed")) {
         std::random_device randDev{};
         std::uniform_int_distribution<int> distrib{1};
-        ctx->rngseed(distrib(randDev));
+        auto seed = distrib(randDev);
+        ctx->rngseed(seed);
+        log_info("Generated random seed: %d\n", seed);
     }
 
     if (vm.count("slack_redist_iter")) {
@@ -270,6 +283,10 @@ void CommandHandler::setupContext(Context *ctx)
         ctx->settings[ctx->id("timing/ignoreLoops")] = true;
     }
 
+    if (vm.count("ignore-rel-clk")) {
+        ctx->settings[ctx->id("timing/ignoreRelClk")] = true;
+    }
+
     if (vm.count("timing-allow-fail")) {
         ctx->settings[ctx->id("timing/allowFail")] = true;
     }
@@ -323,6 +340,10 @@ void CommandHandler::setupContext(Context *ctx)
     if (vm.count("placer-heap-timingweight"))
         ctx->settings[ctx->id("placerHeap/timingWeight")] = std::to_string(vm["placer-heap-timingweight"].as<int>());
 
+    if (vm.count("placer-heap-cell-placement-timeout"))
+        ctx->settings[ctx->id("placerHeap/cellPlacementTimeout")] =
+                std::to_string(std::max(0, vm["placer-heap-cell-placement-timeout"].as<int>()));
+
     if (vm.count("parallel-refine"))
         ctx->settings[ctx->id("placerHeap/parallelRefine")] = true;
 
