@@ -31,8 +31,6 @@
  *   - To make the placer timing-driven, the bound2bound weights are multiplied by (1 + 10 * crit^2)
  */
 
-#ifdef WITH_HEAP
-
 #include "placer_heap.h"
 #include <Eigen/Core>
 #include <Eigen/IterativeLinearSolvers>
@@ -218,7 +216,11 @@ class HeAPPlacer
 
         heap_runs.push_back(all_buckets);
         // The main HeAP placer loop
-        log_info("Running main analytical placer.\n");
+        if (cfg.cell_placement_timeout > 0)
+            log_info("Running main analytical placer, max placement attempts per cell = %d.\n",
+                     cfg.cell_placement_timeout);
+        else
+            log_info("Running main analytical placer.\n");
         while (stalled < 5 && (solved_hpwl <= legal_hpwl * 0.8)) {
             // Alternate between particular bel types and all bels
             for (auto &run : heap_runs) {
@@ -327,7 +329,7 @@ class HeAPPlacer
         bool any_bad_placements = false;
         for (auto bel : ctx->getBels()) {
             CellInfo *cell = ctx->getBoundBelCell(bel);
-            if (!ctx->isBelLocationValid(bel)) {
+            if (!ctx->isBelLocationValid(bel, /* explain_invalid */ true)) {
                 std::string cell_text = "no cell";
                 if (cell != nullptr)
                     cell_text = std::string("cell '") + ctx->nameOf(cell) + "'";
@@ -351,7 +353,7 @@ class HeAPPlacer
         ctx->check();
         lock.unlock_early();
 
-#if !defined(__wasm)
+#if !defined(NPNR_DISABLE_THREADS)
         if (cfg.parallelRefine) {
             if (!parallel_refine(ctx, ParallelRefineCfg(ctx))) {
                 return false;
@@ -359,7 +361,11 @@ class HeAPPlacer
         } else
 #endif
         {
-            if (!placer1_refine(ctx, Placer1Cfg(ctx))) {
+            auto placer1_cfg = Placer1Cfg(ctx);
+            placer1_cfg.hpwl_scale_x = cfg.hpwl_scale_x;
+            placer1_cfg.hpwl_scale_y = cfg.hpwl_scale_y;
+            placer1_cfg.netShareWeight = cfg.netShareWeight;
+            if (!placer1_refine(ctx, placer1_cfg)) {
                 return false;
             }
         }
@@ -377,12 +383,6 @@ class HeAPPlacer
 
     TimingAnalyser tmg;
 
-    struct BoundingBox
-    {
-        // Actual bounding box
-        int x0 = 0, x1 = 0, y0 = 0, y1 = 0;
-    };
-
     dict<IdString, BoundingBox> constraint_region_bounds;
 
     // In some cases, we can't use bindBel because we allow overlap in the earlier stages. So we use this custom
@@ -440,7 +440,7 @@ class HeAPPlacer
                 }
 
                 ctx->bindBel(bel, cell, STRENGTH_USER);
-                if (!ctx->isBelLocationValid(bel)) {
+                if (!ctx->isBelLocationValid(bel, /* explain_invalid */ true)) {
                     IdString bel_type = ctx->getBelType(bel);
                     log_error("Bel \'%s\' of type \'%s\' is not valid for cell "
                               "\'%s\' of type \'%s\'\n",
@@ -669,7 +669,7 @@ class HeAPPlacer
             if (cell->cluster != ClusterId()) {
                 const auto base = cell_locs[cell->name];
                 for (auto child : cluster2cells.at(cell->cluster)) {
-                    if (child->type == cell->type && child != cell)
+                    if (child != cell)
                         chain_size[cell->name]++;
                     Loc offset = ctx->getClusterOffset(child);
                     cell_locs[child->name].x = std::max(0, std::min(max_x, base.x + offset.x));
@@ -862,12 +862,13 @@ class HeAPPlacer
             // Was now placed, ignore
             if (ci->bel != BelId())
                 continue;
-            // log_info("   Legalising %s (%s)\n", top.second.c_str(ctx), ci->type.c_str(ctx));
+            // log_info("   Legalising %s (%s) %d\n", top.second.c_str(ctx), ci->type.c_str(ctx), top.first);
             FastBels::FastBelsData *fb;
             fast_bels.getBelsForCellType(ci->type, &fb);
             int radius = 0;
             int iter = 0;
             int iter_at_radius = 0;
+            int total_iters_for_cell = 0;
             bool placed = false;
             BelId bestBel;
             int best_inp_len = std::numeric_limits<int>::max();
@@ -884,11 +885,11 @@ class HeAPPlacer
             }
 
             while (!placed) {
-
-                // Set a conservative timeout
-                if (iter > std::max(10000, 3 * int(ctx->cells.size())))
-                    log_error("Unable to find legal placement for cell '%s', check constraints and utilisation.\n",
-                              ctx->nameOf(ci));
+                if (cfg.cell_placement_timeout > 0 && total_iters_for_cell > cfg.cell_placement_timeout)
+                    log_error("Unable to find legal placement for cell '%s' after %d attempts, check constraints and "
+                              "utilisation. Use `--placer-heap-cell-placement-timeout` to change the number of "
+                              "attempts.\n",
+                              ctx->nameOf(ci), total_iters_for_cell);
 
                 // Determine a search radius around the solver location (which increases over time) that is clamped to
                 // the region constraint for the cell (if applicable)
@@ -1090,6 +1091,8 @@ class HeAPPlacer
                         break;
                     }
                 }
+
+                total_iters_for_cell++;
             }
         }
         auto endt = std::chrono::high_resolution_clock::now();
@@ -1815,11 +1818,21 @@ PlacerHeapCfg::PlacerHeapCfg(Context *ctx)
     criticalityExponent = ctx->setting<int>("placerHeap/criticalityExponent");
     timingWeight = ctx->setting<int>("placerHeap/timingWeight");
     parallelRefine = ctx->setting<bool>("placerHeap/parallelRefine", false);
+    netShareWeight = ctx->setting<float>("placerHeap/netShareWeight", 0);
 
     timing_driven = ctx->setting<bool>("timing_driven");
     solverTolerance = 1e-5;
     placeAllAtOnce = false;
 
+    int timeout_divisor = ctx->setting<int>("placerHeap/cellPlacementTimeout", 8);
+    if (timeout_divisor > 0) {
+        // Set a conservative default. This is a rather large number and could probably
+        // be shaved down, but for now it will keep the process from running indefinite.
+        cell_placement_timeout = std::max(10000, (int(ctx->cells.size()) * int(ctx->cells.size()) / timeout_divisor));
+    } else {
+        cell_placement_timeout = 0;
+    }
+
     hpwl_scale_x = 1;
     hpwl_scale_y = 1;
     spread_scale_x = 1;
@@ -1827,22 +1840,3 @@ PlacerHeapCfg::PlacerHeapCfg(Context *ctx)
 }
 
 NEXTPNR_NAMESPACE_END
-
-#else
-
-#include "log.h"
-#include "nextpnr.h"
-#include "placer_heap.h"
-
-NEXTPNR_NAMESPACE_BEGIN
-bool placer_heap(Context *ctx, PlacerHeapCfg cfg)
-{
-    log_error("nextpnr was built without the HeAP placer\n");
-    return false;
-}
-
-PlacerHeapCfg::PlacerHeapCfg(Context *ctx) {}
-
-NEXTPNR_NAMESPACE_END
-
-#endif
