@@ -28,6 +28,7 @@
 #include <cmath>
 #include <cstring>
 #include <queue>
+#include <set>
 
 #include "constraints.impl.h"
 #include "fpga_interchange.h"
@@ -45,9 +46,9 @@
 // scope of the header.
 #include <tcl.h>
 
-//#define DEBUG_BINDING
-//#define USE_LOOKAHEAD
-//#define DEBUG_CELL_PIN_MAPPING
+// #define DEBUG_BINDING
+// #define USE_LOOKAHEAD
+// #define DEBUG_CELL_PIN_MAPPING
 
 // Define to enable some idempotent sanity checks for some important
 // operations prior to placement and routing.
@@ -730,7 +731,7 @@ std::vector<std::pair<IdString, std::string>> Arch::getBelAttrs(BelId bel) const
 
 // -----------------------------------------------------------------------
 
-ArcBounds Arch::getRouteBoundingBox(WireId src, WireId dst) const
+BoundingBox Arch::getRouteBoundingBox(WireId src, WireId dst) const
 {
     int dst_tile = dst.tile == -1 ? chip_info->nodes[dst.index].tile_wires[0].tile : dst.tile;
     int src_tile = src.tile == -1 ? chip_info->nodes[src.index].tile_wires[0].tile : src.tile;
@@ -1115,17 +1116,9 @@ std::string Arch::get_part() const
 
 // -----------------------------------------------------------------------
 
-#ifdef WITH_HEAP
 const std::string Arch::defaultPlacer = "heap";
-#else
-const std::string Arch::defaultPlacer = "sa";
-#endif
 
-const std::vector<std::string> Arch::availablePlacers = {"sa",
-#ifdef WITH_HEAP
-                                                         "heap"
-#endif
-};
+const std::vector<std::string> Arch::availablePlacers = {"sa", "heap"};
 
 const std::string Arch::defaultRouter = "router2";
 const std::vector<std::string> Arch::availableRouters = {"router1", "router2"};
@@ -1743,7 +1736,7 @@ void Arch::bindWire(WireId wire, NetInfo *net, PlaceStrength strength)
     refreshUiWire(wire);
 }
 
-bool Arch::checkPipAvailForNet(PipId pip, NetInfo *net) const
+bool Arch::checkPipAvailForNet(PipId pip, const NetInfo *net) const
 {
     NPNR_ASSERT(pip != PipId());
     auto pip_iter = pip_to_net.find(pip);
