@@ -596,7 +596,7 @@ struct Arch : ArchAPI<ArchRanges>
     void unbindPip(PipId pip) final;
 
     bool checkPipAvail(PipId pip) const final;
-    bool checkPipAvailForNet(PipId pip, NetInfo *net) const final;
+    bool checkPipAvailForNet(PipId pip, const NetInfo *net) const final;
 
     NetInfo *getBoundPipNet(PipId pip) const final
     {
@@ -701,7 +701,7 @@ struct Arch : ArchAPI<ArchRanges>
     // -------------------------------------------------
     delay_t estimateDelay(WireId src, WireId dst) const final;
     delay_t predictDelay(BelId src_bel, IdString src_pin, BelId dst_bel, IdString dst_pin) const final;
-    ArcBounds getRouteBoundingBox(WireId src, WireId dst) const final;
+    BoundingBox getRouteBoundingBox(WireId src, WireId dst) const final;
     delay_t getDelayEpsilon() const final { return 20; }
     delay_t getRipupDelayPenalty() const final { return 120; }
     float getDelayNS(delay_t v) const final { return v * 0.001; }
@@ -851,7 +851,7 @@ struct Arch : ArchAPI<ArchRanges>
     }
 
     // Return true whether all Bels at a given location are valid
-    bool isBelLocationValid(BelId bel) const final
+    bool isBelLocationValid(BelId bel, bool explain_invalid = false) const final
     {
         auto iter = tileStatus.find(bel.tile);
         if (iter == tileStatus.end()) {
@@ -896,7 +896,7 @@ struct Arch : ArchAPI<ArchRanges>
     }
 
     CellInfo *getClusterRootCell(ClusterId cluster) const override;
-    ArcBounds getClusterBounds(ClusterId cluster) const override;
+    BoundingBox getClusterBounds(ClusterId cluster) const override;
     Loc getClusterOffset(const CellInfo *cell) const override;
     bool isClusterStrict(const CellInfo *cell) const override;
     bool normal_cluster_placement(const Context *, const Cluster &, const ClusterPOD &, CellInfo *, BelId,
