@@ -140,10 +140,17 @@ NPNR_PACKED_STRUCT(struct PartnumberPOD {
     uint32_t speed_id;
 });
 
+NPNR_PACKED_STRUCT(struct PinPOD {
+    uint16_t index_id;
+    uint16_t loc_id;
+    uint32_t num_cfgs;
+    RelPtr<uint32_t> cfgs;
+});
+
 NPNR_PACKED_STRUCT(struct PackagePOD {
     uint32_t name_id;
     uint32_t num_pins;
-    RelPtr<PairPOD> pins;
+    RelPtr<PinPOD> pins;
 });
 
 NPNR_PACKED_STRUCT(struct VariantPOD {
@@ -220,6 +227,7 @@ struct BelInfo
     std::map<IdString, std::string> attrs;
     CellInfo *bound_cell;
     dict<IdString, PinInfo> pins;
+    std::vector<IdString> pin_cfgs;
     DecalXY decalxy_active, decalxy_inactive;
     int x, y, z;
     bool gb;
@@ -275,6 +283,12 @@ struct ArchRanges : BaseArchRanges
     using GroupGroupsRangeT = const std::vector<GroupId> &;
 };
 
+enum class PLL // fixed PLL locations
+{
+    left,
+    right
+};
+
 struct Arch : BaseArch<ArchRanges>
 {
     std::string family;
@@ -387,6 +401,11 @@ struct Arch : BaseArch<ArchRanges>
     PortType getBelPinType(BelId bel, IdString pin) const override;
     std::vector<IdString> getBelPins(BelId bel) const override;
     std::array<IdString, 1> getBelPinsForCellPin(const CellInfo *cell_info, IdString pin) const override;
+    // Placement validity checks
+    virtual bool isValidBelForCellType(IdString cell_type, BelId bel) const override
+    {
+        return cell_type == id_DUMMY_CELL || cell_type == this->getBelType(bel);
+    }
 
     WireId getWireByName(IdStringList name) const override;
     IdStringList getWireName(WireId wire) const override;
@@ -439,7 +458,7 @@ struct Arch : BaseArch<ArchRanges>
     uint32_t getDelayChecksum(delay_t v) const override { return 0; }
     bool getBudgetOverride(const NetInfo *net_info, const PortRef &sink, delay_t &budget) const override;
 
-    ArcBounds getRouteBoundingBox(WireId src, WireId dst) const override;
+    BoundingBox getRouteBoundingBox(WireId src, WireId dst) const override;
 
     bool pack() override;
     bool place() override;
@@ -451,7 +470,7 @@ struct Arch : BaseArch<ArchRanges>
     // Get the TimingClockingInfo of a port
     TimingClockingInfo getPortClockingInfo(const CellInfo *cell, IdString port, int index) const override;
 
-    bool isBelLocationValid(BelId bel) const override;
+    bool isBelLocationValid(BelId bel, bool explain_invalid = false) const override;
 
     static const std::string defaultPlacer;
     static const std::vector<std::string> availablePlacers;
@@ -467,14 +486,26 @@ struct Arch : BaseArch<ArchRanges>
     void fix_longwire_bels();
     void pre_pack(Context *ctx);
     void post_pack(Context *ctx);
+    void pre_route(Context *ctx);
+    void post_route(Context *ctx);
     void auto_longwires();
+    void add_pllvr_ports(DatabasePOD const *db, BelsPOD const *bel, IdString belname, int row, int col);
+    void add_rpll_ports(DatabasePOD const *db, BelsPOD const *bel, IdString belname, int row, int col);
+    void fix_pll_nets(Context *ctx);
+    bool is_GCLKT_iob(const CellInfo *cell);
+    void bind_pll_to_bel(CellInfo *ci, PLL loc);
+
+    void mark_used_hclk(Context *ctx);
+    IdString apply_local_aliases(int row, int col, const DatabasePOD *db, IdString &wire);
+
+    WireId get_make_port_wire(const DatabasePOD *db, const BelsPOD *bel, int row, int col, IdString port);
 
     GowinGlobalRouter globals_router;
     void mark_gowin_globals(Context *ctx);
     void route_gowin_globals(Context *ctx);
 
     // chip db version
-    unsigned int const chipdb_version = 1;
+    unsigned int const chipdb_version = 2;
 
     std::vector<IdString> cell_types;
 
@@ -494,20 +525,29 @@ struct Arch : BaseArch<ArchRanges>
 
     // Permissible combinations of modes in a single slice
     std::map<const IdString, IdString> dff_comp_mode;
+
+    // max global clock wires
+    int max_clock;
 };
 
 // Bels Z range
 namespace BelZ {
 enum
 {
-    mux_0_z = 10,     // start Z for the MUX2LUT5 bels
-    iologic_0_z = 20, // start Z for the IOLOGIC bels
-    lutram_0_z = 30,  // start Z for the IOLOGIC bels
-    vcc_0_z = 277,    // virtual VCC bel Z
-    gnd_0_z = 278,    // virtual VSS bel Z
-    osc_z = 280,      // Z for the oscillator bels
-    bufs_0_z = 281,   // Z for long wire buffer bel
-    free_z = 289      // Must be the last, one can use z starting from this value, adjust accordingly.
+    ioba_z = 0,      // IOBA
+    iobb_z = 1,      // IOBB
+    mux_0_z = 10,    // start Z for the MUX2LUT5 bels
+    lutram_0_z = 30, // start Z for the LUTRAM bels
+    vcc_0_z = 277,   // virtual VCC bel Z
+    gnd_0_z = 278,   // virtual VSS bel Z
+    osc_z = 280,     // Z for the oscillator bels
+    bufs_0_z = 281,  // Z for long wire buffer bel
+    pll_z = 289,     // PLL
+    pllvr_z = 290,   // PLLVR
+    iologic_z = 291, // IOLOGIC
+    oser16_z = 293,  // OSER16
+    ides16_z = 294,  // IDES16
+    free_z = 295     // Must be the last, one can use z starting from this value, adjust accordingly.
 };
 }
 
