@@ -333,7 +333,7 @@ struct Arch : BaseArch<ArchRanges>
     PortType getBelPinType(BelId bel, IdString pin) const override { return bel_data(bel).pins.at(pin).dir; }
     std::vector<IdString> getBelPins(BelId bel) const override;
 
-    bool isBelLocationValid(BelId bel) const override;
+    bool isBelLocationValid(BelId bel, bool explain_invalid = false) const override;
 
     void bindBel(BelId bel, CellInfo *cell, PlaceStrength strength) override
     {
@@ -408,7 +408,7 @@ struct Arch : BaseArch<ArchRanges>
         return BaseArch::checkPipAvail(pip);
     }
 
-    bool checkPipAvailForNet(PipId pip, NetInfo *net) const override
+    bool checkPipAvailForNet(PipId pip, const NetInfo *net) const override
     {
         if (is_pip_blocked(pip))
             return false;
@@ -425,7 +425,7 @@ struct Arch : BaseArch<ArchRanges>
     delay_t getDelayFromNS(float ns) const override { return delay_t(ns * 1000.0f); };
     uint32_t getDelayChecksum(delay_t v) const override { return v; };
 
-    ArcBounds getRouteBoundingBox(WireId src, WireId dst) const override;
+    BoundingBox getRouteBoundingBox(WireId src, WireId dst) const override;
 
     TimingPortClass getPortTimingClass(const CellInfo *cell, IdString port,
                                        int &clockInfoCount) const override;                                // delay.cc
