@@ -48,6 +48,7 @@ po::options_description NexusCommandHandler::getArchOptions()
 {
     po::options_description specific("Architecture specific options");
     specific.add_options()("device", po::value<std::string>(), "device name");
+    specific.add_options()("list-devices", "list all supported device names");
     specific.add_options()("fasm", po::value<std::string>(), "fasm file to write");
     specific.add_options()("pdc", po::value<std::string>(), "physical constraints file");
     specific.add_options()("no-post-place-opt", "disable post-place repacking (debugging use only)");
@@ -73,6 +74,10 @@ void NexusCommandHandler::customBitstream(Context *ctx)
 std::unique_ptr<Context> NexusCommandHandler::createContext(dict<std::string, Property> &values)
 {
     ArchArgs chipArgs;
+    if (vm.count("list-devices")) {
+        Arch::list_devices();
+        exit(0);
+    }
     if (!vm.count("device")) {
         log_error("device must be specified on the command line (e.g. --device LIFCL-40-9BG400CES)\n");
     }
